/************************************************************************************
 *  File:     main.c
 *  Purpose:  Cortex-M4 main file.
 *            Replace with your code.
 *  Date:     05 July 2013
 *  Info:     If __NO_SYSTEM_INIT is defined in the Build options,
 *            the startup code will not branch to SystemInit()
 *            and the function can be removed
 ************************************************************************************/
#include <stdio.h>

#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "utils/uartstdio.h"
#include "math.h"

#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "main.h"


/*
* Function prototypes..
*/
static void prvMainCTRLTask(void *pvParameters);
static void prvDummyTask(void *pvParameters);

/*
* Variables
*/
xSemaphoreHandle semUART;




#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}

#endif

#ifndef __NO_SYSTEM_INIT
void SystemInit()
{}
#endif


/*********************************************************************
 *
 *  main()
 *********************************************************************/
void main()
{
    //
    // Set the clocking to run at 50 MHz from the PLL (with SYSCTL_SYSDIV_4).
    // PLL is fixed at 400Mhz, and is divided down by 2, so
    // to run M4 at 80Mhz, use SYSCTL_SYSDIV_2_5.
    // REMEMBER To change FreeRTOSConfig.h - configCPU_CLOCK_HZ if clock speed is
    // changed.
    ROM_SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_XTAL_16MHZ |
                       SYSCTL_OSC_MAIN);

    // Enable floating point unit
    ROM_FPUEnable();

    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    ROM_GPIOPinTypeGPIOOutput(GPIO_PORTA_BASE, GPIO_PIN_7);  // GPIO_PORTA_AHB_BASE for better performance, GPIO_PORTA_BASE for better power savings
    ROM_GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_7, 0);


    // configure some portb pins for debugging
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    ROM_GPIOPinTypeGPIOOutput(GPIO_PORTB_BASE, GPIO_PIN_1 | GPIO_PIN_0);
    ROM_GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_1 | GPIO_PIN_0, 0);


    //
    // Create a mutex to guard the UART.
    //
    semUART = xSemaphoreCreateMutex();

    // Create a main task here which will launch other tasks..
    xTaskCreate(prvMainCTRLTask, ( signed portCHAR * ) "MainCTRL", configMINIMAL_STACK_SIZE, NULL, mainTASK_PRIORITY, NULL);
    /* Start the scheduler. */
    vTaskStartScheduler();

    /* If all is well then this line will never be reached.  If it is reached
    then it is likely that there was insufficient (FreeRTOS) heap memory space
    to create the idle task.  This may have been trapped by the malloc() failed
    hook function, if one is configured. */
    for( ;; );
}


/*
* Main task
*/
static void prvMainCTRLTask(void *pvParameters)
{
    int cnt;
    float afloat, bfloat;

    //
    // Initialize the UART and configure it for 115,200, 8-N-1 operation.
    // This uses USB virtual comport from ICDI..
    //
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    GPIOPinConfigure(GPIO_PA0_U0RX);
    GPIOPinConfigure(GPIO_PA1_U0TX);
    ROM_GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);
    UARTStdioInit(0);

    xTaskCreate(prvDummyTask, ( signed portCHAR * ) "MainCTRL", configMINIMAL_STACK_SIZE * 2, NULL, dummyTASK_PRIORITY, NULL);

    afloat = 0.3;
    bfloat = 0.4;
    atan2f(afloat, bfloat);

    cnt = 0;
    while(1)
    {
        cnt++;

        xSemaphoreTake(semUART, portMAX_DELAY);
        UARTprintf("\n\n%s running!\n", __FUNCTION__);
        xSemaphoreGive(semUART);

        ROM_GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_0, GPIO_PIN_0);
        vTaskDelay(100/portTICK_RATE_MS);  // delay
        ROM_GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_0, 0);
    }
}


static void prvDummyTask(void *pvParameters)
{
    int cnt;
    float afloat, bfloat;
    float ans;
    char buffer[128];


    cnt = 0;
    while(1)
    {
        cnt++;
        afloat = 0.3;
        bfloat = 0.4;
        ans = atan2f(afloat, bfloat);

        sprintf(buffer, "\n\n%s running! ans=%f", __FUNCTION__, ans);

        xSemaphoreTake(semUART, portMAX_DELAY);
        UARTprintf("%s", buffer);
        xSemaphoreGive(semUART);

        //                                 MASK        VALUE to write (according to MASK)
        ROM_GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_1, GPIO_PIN_1);
        vTaskDelay(150/portTICK_RATE_MS);  // delay
        ROM_GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_1, 0);
    }
}


/***************************************************************************/
/************ FreeRTOS Hooks ***********************************************/

void vApplicationStackOverflowHook( xTaskHandle pxTask, signed char *pcTaskName )
{
    ( void ) pcTaskName;
    ( void ) pxTask;

    /* Run time stack overflow checking is performed if
    configconfigCHECK_FOR_STACK_OVERFLOW is defined to 1 or 2.  This hook
    function is called if a stack overflow is detected. */
    for( ;; );
}


void vApplicationMallocFailedHook( void )
{
    /* Called if a call to pvPortMalloc() fails because there is insufficient
    free memory available in the FreeRTOS heap.  pvPortMalloc() is called
    internally by FreeRTOS API functions that create tasks, queues or
    semaphores. */
    for( ;; );
}


void vApplicationTickHook( void )
{
    static unsigned int toggle = 0;
    if (toggle)
    {
        toggle = 0;
        ROM_GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_7, GPIO_PIN_7);
    }
    else
    {
        toggle = 1;
        ROM_GPIOPinWrite(GPIO_PORTA_BASE, GPIO_PIN_7, 0);
    }
}


void vApplicationIdleHook( void )
{
}



